close all
clear
clc

addpath('/opt/openEMS/share/openEMS/matlab');
addpath('/opt/openEMS/share/CSXCAD/matlab');
addpath('/opt/openEMS/share/hyp2mat/matlab');
addpath('/opt/openEMS/share/CTB/matlab');
physical_constants; #import physical_constants

###############################
## PREPARE SIMULATION MODEL
###############################

#variables

fmin = 0.1e9;
fmax = 10e9;
impedance = 50;
eps_r = 4.5; #FR-4 permability
tan_loss = 0.02; #FR-4 losses (as tangens)
kap = tan_loss*2*pi*((fmin*fmax)^0.5)*EPS0*eps_r; #FR-4 losses (as kappa)
res = c0 / (fmax*sqrt(eps_r)) / 60;
res_f = res / 3;

subst_h = 1e-3;
boxsize_x = 10e-3;
boxsize_y = 9e-3;
boxsize_z = subst_h*5;
feedline_w = 0.2e-3;
stub_l = 2.0e-3;
stub_cw = 1e-3;
stub_alpha = 120;
c_ref = 1.016e-12;

#initiate and setup simulation structures and parameters
FDTD = InitFDTD('EndCriteria', 1e-4);
FDTD = SetGaussExcite(FDTD, fmin, fmax);
BC = {'PML_8' 'PML_8' 'PML_8' 'PML_8' 'PEC' 'PML_8'};
FDTD = SetBoundaryCond(FDTD, BC);
CSX = InitCSX();

#definitions of used materials
CSX = AddMaterial(CSX, 'FR4');
CSX = SetMaterialProperty(CSX, 'FR4', 'Epsilon', eps_r, 'Kappa', kap);
CSX = AddMetal(CSX,'Metal', 56e6, 16e-6);


#geometry
#stub
CSX = AddMRStub(CSX, 'Metal', 100, stub_cw, stub_l, stub_alpha, 1, -90, [0 0 1], [0 -feedline_w/2 subst_h]);
CSX = AddMRStub(CSX, 'Metal', 100, stub_cw, stub_l, stub_alpha, 1, 90, [0 0 1], [0 feedline_w/2 subst_h]);
CSX = AddBox(CSX, 'Metal', 100, [-stub_cw/2, -feedline_w/2, subst_h], [stub_cw/2, feedline_w/2, subst_h]);
#substrate
start = [-boxsize_x/2, -boxsize_y/2, 0];
stop = [boxsize_x/2, boxsize_y/2, subst_h];
CSX = AddBox(CSX, 'FR4', 10, start, stop);
#setup mesh
mesh.x = SmoothMeshLines([-boxsize_x/2, -2*feedline_w, boxsize_x/2], res_f, 1.5);
mesh.y = SmoothMeshLines([-boxsize_y/2, boxsize_y/2], res_f, 1.5);
mesh.z = SmoothMeshLines([0, subst_h, 2*subst_h], res_f, 1.5);
mesh.z = SmoothMeshLines([mesh.z, boxsize_z], res, 1.5);
CSX = DefineRectGrid(CSX, 1, mesh);
#add ports
CSX = AddMetal( CSX, 'PEC' );
portstart = [ mesh.x(1), -feedline_w/2, subst_h];
portstop  = [ 0,  feedline_w/2, 0];
[CSX,port{1}] = AddMSLPort( CSX, 999, 1, 'PEC', portstart, portstop, 0, [0 0 -1], 'ExcitePort', 'excite', 'FeedShift', 10*res, 'MeasPlaneShift', boxsize_x/2-stub_cw/2-0.2e-3);

###############################
## PREPARE AND SIMULATE
###############################

#set files, paths, write data, show geometry, run the simulator
confirm_recursive_rmdir(0); #don't want asking when remomoving directory
sim_path = 'sim_path_s1';
sim_file = 'sim_file.xml';
rmdir(sim_path, 's');
mkdir(sim_path);
#save structures with data about simulation to file
WriteOpenEMS([sim_path '/' sim_file], FDTD, CSX);
#run a geometry viewer
CSXGeomPlot([sim_path '/' sim_file]);
#simulate
RunOpenEMS(sim_path, sim_file);


###############################
## PROCESS RESULTS
###############################

f = linspace(fmin, fmax, 2000);
port = calcPort(port, sim_path, f, 'RefImpedance', impedance);
z_stub = port{1}.uf.tot ./ port{1}.if.tot;
z_nom = 1 ./ (2*pi*f*c_ref);
figure(1);
hold on;
loglog(f/1e9, abs(z_stub), 'k-', 'LineWidth', 2);
loglog(f/1e9, abs(z_nom), 'y-', 'LineWidth', 2);
grid on;
figure(2);
hold on;
semilogx(f/1e9, (arg(z_stub).*(180/pi)), 'k-', 'LineWidth', 2);
semilogx(f/1e9, (arg(z_nom).*(180/pi)), 'b-', 'LineWidth', 2);
grid on;
